@@ -1,10 +1,10 @@
-  /*
+/*
  * scrm is an implementation of the Sequential-Coalescent-with-Recombination Model.
- * 
+ *
  * Copyright (C) 2013, 2014 Paul R. Staab, Sha (Joe) Zhu, Dirk Metzler and Gerton Lunter
- * 
+ *
  * This file is part of scrm.
- * 
+ *
  * scrm is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -32,7 +32,7 @@ Forest::Forest(Model* model, RandomGenerator* random_generator) {
 }
 
 // Sets member variable to default values
-void Forest::initialize(Model* model, 
+void Forest::initialize(Model* model,
                         RandomGenerator* rg) {
 
   model->resetTime();
@@ -49,9 +49,10 @@ void Forest::initialize(Model* model,
 
   this->coalescence_finished_ = true;
 
-  this->contemporaries_ = ContemporariesContainer(model->population_number(), 
+  this->contemporaries_ = ContemporariesContainer(model->population_number(),
                                                   model->sample_size(),
                                                   rg);
+
   tmp_event_time_ = -1;
 }
 
@@ -59,7 +60,7 @@ void Forest::initialize(Model* model,
  * @brief Copy constructor for forest
  *
  * This creates a copy of an Forest. It only produces valid results
- * when there is no ongoing coalescence event (e.g. when we are not 
+ * when there is no ongoing coalescence event (e.g. when we are not
  * inside of sampleNextGenealogy()).
  *
  * Also, both copies of it share the model and the random generator,
@@ -68,7 +69,7 @@ void Forest::initialize(Model* model,
  *
  * @param current_forest Forest that needs to be duplicated
  */
-Forest::Forest(const Forest &current_forest) { 
+Forest::Forest(const Forest &current_forest) {
   if (!current_forest.coalescence_finished_) {
     throw std::logic_error("Can not copy forest during an ongoing coalescence");
   }
@@ -93,10 +94,12 @@ Forest::Forest(const Forest &current_forest) {
   }
 
   // Set initial values for temporary variables
-  this->contemporaries_ = ContemporariesContainer(model().population_number(), 
+  this->contemporaries_ = ContemporariesContainer(model().population_number(),
                                                   model().sample_size(),
                                                   random_generator());
-  this->tmp_event_time_ = -1; 
+
+  this->tmp_event_time_ = this->getTMRCA(false); // Disable buffer for next genealogy.
+
   this->coalescence_finished_ = true;
 
   dout<<"  #################### check copied forest ###############"<<std::endl;
@@ -108,12 +111,12 @@ Forest::Forest(const Forest &current_forest) {
 }
 
 
-/** 
+/**
  * function that cuts a subtree out of a tree of the forest and reinserts it as
  * a separate tree.
  *
  * This is primarily used to cut the subtree below an recombination
- * away. 
+ * away.
  *
  * \param cut_point   A TreePoint marking the top of the subtree to cut.
  * \return            The root of the now separated subtree.
@@ -125,7 +128,7 @@ Node* Forest::cut(const TreePoint &cut_point) {
 
   //The new end of the old branch after the cut
   Node* new_leaf = nodes()->createNode(cut_point.height());
-  
+
   if ( !cut_point.base_node()->local() )
     new_leaf->make_nonlocal(cut_point.base_node()->last_update());
   else
@@ -156,7 +159,7 @@ Node* Forest::cut(const TreePoint &cut_point) {
   new_root->set_first_child(cut_point.base_node());
 
   // Set invariants of new root
-  new_root->set_length_below(cut_point.base_node()->length_below() + 
+  new_root->set_length_below(cut_point.base_node()->length_below() +
                              cut_point.relative_height() );
   new_root->set_samples_below(cut_point.base_node()->samples_below() );
 
@@ -177,7 +180,7 @@ Node* Forest::cut(const TreePoint &cut_point) {
 
 
 /**
- * Function to update the invariants (local, samples_below, length_below) 
+ * Function to update the invariants (local, samples_below, length_below)
  * of a 'node' and all of its (grand-)parents. Also sets local_root_ if it
  * encounters it. Never makes non-local nodes local, only the other way round.
  *
@@ -192,11 +195,11 @@ Node* Forest::cut(const TreePoint &cut_point) {
  *  \param invariants_only If true, it only updates the nodes invariants, but
  *                    does not make nodes non-local and change the local root.
  */
-void Forest::updateAbove(Node* node, bool above_local_root, 
+void Forest::updateAbove(Node* node, bool above_local_root,
                          const bool &recursive, const bool &invariants_only) {
 
   //dout << "Updating: " << node << " above_local_root: " << above_local_root << std::endl;
-  
+
   // Fast forward above local root because this part is fairly straight forward
   if (above_local_root) {
     // Assure that everything is non-local
@@ -208,7 +211,7 @@ void Forest::updateAbove(Node* node, bool above_local_root,
         set_primary_root(node);
       }
       return;
-    } 
+    }
     if ( recursive ) updateAbove(node->parent(), true, true);
     return;
   }
@@ -237,7 +240,7 @@ void Forest::updateAbove(Node* node, bool above_local_root,
   }
   assert( length_below >= 0 );
 
-  // Update whether the node is local or not 
+  // Update whether the node is local or not
   if (!invariants_only) {
     if (samples_below == 0) {
       if ( node->local() ) node->make_nonlocal(current_rec());
@@ -246,7 +249,7 @@ void Forest::updateAbove(Node* node, bool above_local_root,
       if ( node->local() ) node->make_nonlocal(current_rec());
 
       // Are we the local root?
-      if (node->countChildren() == 2 && 
+      if (node->countChildren() == 2 &&
           l_child->samples_below() > 0 && h_child->samples_below() > 0) {
         set_local_root(node);
       }
@@ -258,7 +261,7 @@ void Forest::updateAbove(Node* node, bool above_local_root,
   }
 
   // If nothing changed, we also don't need to update the tree further above...
-  if (samples_below == node->samples_below() && 
+  if (samples_below == node->samples_below() &&
       areSame(length_below, node->length_below()) ) {
     return;
   }
@@ -285,6 +288,7 @@ void Forest::buildInitialTree() {
   assert(this->segment_count() == 0);
   assert(this->rec_bases_.size() == 1);
   assert(this->model().getCurrentSequencePosition() == 0.0);
+  assert(this->contemporaries()->empty());
   this->set_next_base(0.0);
   ++current_rec_;
 
@@ -310,7 +314,7 @@ void Forest::buildInitialTree() {
     if (new_leaf->height() == 0.0) last_added_node = new_leaf;
     dout << "* starting coalescences" << std::endl;
 
-    //Coalesces the separate tree into the main tree
+    // Coalesces the separate tree into the main tree
     this->sampleCoalescences(new_leaf);
     dout << "* * Tree:" << std::endl;
 
@@ -318,6 +322,7 @@ void Forest::buildInitialTree() {
     assert(this->checkLeafsOnLocalTree());
     assert(this->printTree());
     assert(this->printNodes());
+    assert(this->coalescence_finished());
   }
   this->sampleNextBase();
   dout << "Next Sequence position: " << this->next_base() << std::endl;
@@ -385,12 +390,12 @@ TreePoint Forest::samplePoint(Node* node, double length_left) const {
   double tmp = node->first_child()->height_above() + node->first_child()->length_below();
   if ( length_left <= tmp )
     return samplePoint(node->first_child(), length_left);
-  else 
+  else
     return samplePoint(node->second_child(), length_left - tmp);
 }
 /* Alternative inefficient implementation
 TreePoint Forest::samplePoint(Node* node, double length_left) {
- length_left = random_generator()->sample() * local_tree_length();  
+ length_left = random_generator()->sample() * local_tree_length();
  for (auto ni = nodes()->iterator(); ni.good(); ++ni) {
    if (!(*ni)->local()) continue;
    if (length_left < (*ni)->height_above()) return TreePoint(*ni, length_left, true);
@@ -402,7 +407,7 @@ TreePoint Forest::samplePoint(Node* node, double length_left) {
 
 
 
-/** 
+/**
  * Function to modify the tree after we encountered a recombination on the
  * sequence. Also samples a place for this recombination on the tree, marks the
  * branch above as non-local (and updates invariants) if needed, cuts the
@@ -452,6 +457,7 @@ void Forest::sampleNextGenealogy() {
   assert( this->checkTree() );
   assert( this->printTree() );
   assert( this->printNodes() );
+  assert( this->coalescence_finished() );
 
   this->sampleNextBase();
   dout << "Next Sequence position: " << this->next_base() << std::endl;
@@ -459,7 +465,7 @@ void Forest::sampleNextGenealogy() {
 }
 
 
-/** 
+/**
  * Function for doing a coalescence.
  *
  * \param start_node The node at which the coalescence starts. Must be the root
@@ -469,28 +475,29 @@ void Forest::sampleCoalescences(Node *start_node) {
   assert( start_node->is_root() );
   // We can have one or active local nodes: If the coalescing node passes the
   // local root, it also starts a coalescence.
-  set_active_node(0, start_node);
-  set_active_node(1, this->local_root());
+  if (start_node->height() > this->local_root()->height()) {
+    set_active_node(0, this->local_root());
+    set_active_node(1, start_node);
+  } else {
+    set_active_node(0, start_node);
+    set_active_node(1, this->local_root());
+  }
+  assert ( active_node(0)->height() <= active_node(1)->height() );
 
   // Initialize Temporary Variables
-  tmp_event_ = Event(start_node->height());
+  tmp_event_ = Event(active_node(0)->height());
   coalescence_finished_ = false;
 
-  // This assertion needs an exception for building the initial tree
-  assert ( current_rec() == 1 || 
-           active_node(1)->in_sample() || 
-           start_node->height() <= active_node(1)->height() );
-
   // Only prune every second round
-  for (TimeIntervalIterator ti(this, start_node); ti.good(); ++ti) {
+  for (TimeIntervalIterator ti(this, active_node(0)); ti.good(); ++ti) {
 
     dout << "* * Time interval: " << (*ti).start_height() << " - "
          << (*ti).end_height() << " (Last event at " << tmp_event_.time() << ")" << std::endl;
 
-    // Assert that we don't accidentally jump in time 
+    // Assert that we don't accidentally jump in time
     assert( tmp_event_.time() < 0 || tmp_event_.time() == (*ti).start_height() );
 
-    // Update States & Rates (see their declaration for explanation); 
+    // Update States & Rates (see their declaration for explanation);
     states_[0] = getNodeState(active_node(0), (*ti).start_height());
     states_[1] = getNodeState(active_node(1), (*ti).start_height());
 
@@ -503,10 +510,10 @@ void Forest::sampleCoalescences(Node *start_node) {
 
     // Some debug checks
     dout << "* * * Active Nodes: a0:" << active_node(0) << ":s" << states_[0]
-        << "(p" << active_node(0)->population() << ")" 
-        << " a1:" << active_node(1) << ":s" << states_[1] 
+        << "(p" << active_node(0)->population() << ")"
+        << " a1:" << active_node(1) << ":s" << states_[1]
         << "(p" << active_node(1)->population() << ")" << std::endl
-        << "* * * Total Rates: " << rates_[0] << " " 
+        << "* * * Total Rates: " << rates_[0] << " "
         << rates_[1] << " " << rates_[2] << std::endl;
 
     assert( active_node(0) != active_node(1) );
@@ -517,7 +524,7 @@ void Forest::sampleCoalescences(Node *start_node) {
     assert( states_[1] == 1 || active_node(1)->parent_height() >= tmp_event_.time() );
     assert( states_[0] != 2 || !active_node(0)->local() );
     assert( states_[1] != 2 || !active_node(1)->local() );
-    
+
     assert( active_node(0)->first_child() == NULL  || active_node(0)->first_child()->local() ||
             active_node(0)->second_child() == NULL || active_node(0)->second_child()->local() );
     assert( active_node(1)->first_child() == NULL  || active_node(1)->first_child()->local() ||
@@ -530,16 +537,14 @@ void Forest::sampleCoalescences(Node *start_node) {
     assert( tmp_event_.isNoEvent() || tmp_event_.time() <= (*ti).end_height() );
 
 
-    // Go on if nothing happens in this time interval
+    // Implement the event
     if ( tmp_event_.isNoEvent() ) {
       this->implementNoEvent(*ti, coalescence_finished_);
-      if (coalescence_finished_) return;
     }
 
-    // First take care of pairwise coalescence
     else if ( tmp_event_.isPwCoalescence() ) {
       this->implementPwCoalescence(active_node(0), active_node(1), tmp_event_.time());
-      return;
+      this->coalescence_finished_ = true;
     }
 
     else if ( tmp_event_.isRecombination() ) {
@@ -548,17 +553,17 @@ void Forest::sampleCoalescences(Node *start_node) {
 
     else if ( tmp_event_.isMigration() ) {
       this->implementMigration(tmp_event_, true, ti);
-      assert( this->printTree() );
     }
 
     else if ( tmp_event_.isCoalescence() ) {
       this->implementCoalescence(tmp_event_, ti);
       assert( checkInvariants(tmp_event_.node()) );
-      if (coalescence_finished_) return;
       assert( this->printTree() );
     }
+
+    if (coalescence_finished()) return;
   }
-}  
+}
 
 
 void Forest::calcRates(const TimeInterval &ti) {
@@ -572,8 +577,8 @@ void Forest::calcRates(const TimeInterval &ti) {
   if (states_[0] == 1) {
     // coalescing or migrating
     rates_[0] += model().total_migration_rate(active_node(0)->population());
-    if (model().growth_rate(active_node(0)->population()) == 0.0) 
-      rates_[0] += calcCoalescenceRate(active_node(0)->population(), ti); 
+    if (model().growth_rate(active_node(0)->population()) == 0.0)
+      rates_[0] += calcCoalescenceRate(active_node(0)->population(), ti);
     else {
       // exponential growth -- assign this node to timeline 1
       rates_[1] += calcCoalescenceRate(active_node(0)->population(), ti);
@@ -582,10 +587,10 @@ void Forest::calcRates(const TimeInterval &ti) {
   }
   else if (states_[0] == 2) {
     // recombining
-    rates_[0] += calcRecombinationRate(active_node(0));    
+    rates_[0] += calcRecombinationRate(active_node(0));
   }
 
-  // The second node is a bit more complicated 
+  // The second node is a bit more complicated
   if (states_[1] == 1) {
     // coalescing or migrating
     rates_[0] += model().total_migration_rate(active_node(1)->population());
@@ -593,9 +598,9 @@ void Forest::calcRates(const TimeInterval &ti) {
       // No Growth => Normal time
       rates_[0] += calcCoalescenceRate(active_node(1)->population(), ti);
 
-      if (states_[0] == 1 && active_node(0)->population() == active_node(1)->population()) { 
+      if (states_[0] == 1 && active_node(0)->population() == active_node(1)->population()) {
         // Also add rates for pw coalescence
-        rates_[0] += calcPwCoalescenceRate(active_node(1)->population(), ti); 
+        rates_[0] += calcPwCoalescenceRate(active_node(1)->population(), ti);
       }
     }
     else {
@@ -604,9 +609,9 @@ void Forest::calcRates(const TimeInterval &ti) {
         // Coalescing or migrating; and we can use the timeline of the first node
         rates_[1] += calcCoalescenceRate(active_node(1)->population(), ti);
         // And must add pw coalescence again
-        rates_[1] += calcPwCoalescenceRate(active_node(1)->population(), ti); 
+        rates_[1] += calcPwCoalescenceRate(active_node(1)->population(), ti);
         active_nodes_timelines_[1] = 1;
-      } 
+      }
       else {
 	      // No chance of a pairwise coalescence, but there is growth.
         // We might need our own timeline (This could be made more efficient if both populations have
@@ -624,33 +629,33 @@ void Forest::calcRates(const TimeInterval &ti) {
   assert(rates_[0] >= 0);
   assert(rates_[1] >= 0);
   assert(rates_[2] >= 0);
-}  
+}
 
 
 /**
  * Samples if an event (coalescence, recombination or migration of active nodes)
- * happens in the current TimeInterval or not. 
+ * happens in the current TimeInterval or not.
  *
  * In particular requires that the 'temporary' forest members samples_, rates_
- * and active_nodes_ are set correctly beforehand.  
+ * and active_nodes_ are set correctly beforehand.
  *
  * \param ti The current time interval
  * \returns the event that has happened (can also be a "NoEvent" event)
  */
 void Forest::sampleEvent(const TimeInterval &ti, double &event_time, Event &return_event) const {
   event_time = -1;
-  size_t event_line = -1;
+  size_t event_line = std::numeric_limits<size_t>::max();
 
   // Sample on which time and time line the event happens (if any)
   for (size_t i = 0; i < 3; ++i) {
     if (rates_[i] == 0.0) continue;
-    selectFirstTime(random_generator()->sampleExpoExpoLimit(rates_[i], getTimeLineGrowth(i), ti.length()), 
+    selectFirstTime(random_generator()->sampleExpoExpoLimit(rates_[i], getTimeLineGrowth(i), ti.length()),
                     i, event_time, event_line);
   }
 
-  // Correct the time from relative to the time interval to absolute 
+  // Correct the time from relative to the time interval to absolute
   if (event_time != -1) event_time += ti.start_height();
-  assert( (event_time == -1) || 
+  assert( (event_time == -1) ||
          (ti.start_height() <= event_time && event_time <= ti.end_height()) );
 
   assert( (event_line == -1 && event_time == -1) || (event_line != -1 && event_time != -1));
@@ -660,16 +665,16 @@ void Forest::sampleEvent(const TimeInterval &ti, double &event_time, Event &retu
 
 
 /**
- * Given that an event has happened, this function samples the events type. 
- * 
- * In particular requires that the 'temporary' forest members samples_, rates_, 
- * active_nodes_, and nodes_timelines_ are set correctly beforehand.  
+ * Given that an event has happened, this function samples the events type.
+ *
+ * In particular requires that the 'temporary' forest members samples_, rates_,
+ * active_nodes_, and nodes_timelines_ are set correctly beforehand.
  */
-void Forest::sampleEventType(const double time, const size_t time_line, 
+void Forest::sampleEventType(const double time, const size_t time_line,
                              const TimeInterval &ti, Event &event) const {
   event = Event(time);
 
-  if ( time_line != -1 && rates_[time_line] == 0.0 ) {
+  if ( time_line != std::numeric_limits<size_t>::max() && rates_[time_line] == 0.0 ) {
     throw std::logic_error("An event with rate 0 has happened!");
   }
 
@@ -683,13 +688,13 @@ void Forest::sampleEventType(const double time, const size_t time_line,
     // Only Nodes in state 1 or 2 can do something
     if ( states_[i] == 0 ) continue;
 
-    // Coalescence can occur on all time lines  
+    // Coalescence can occur on all time lines
     if (states_[i] == 1 && active_nodes_timelines_[i] == time_line) {
       sample -= calcCoalescenceRate(active_node(i)->population(), ti);
-      if (sample <= 0.0) return event.setToCoalescence(active_node(i), i); 
+      if (sample <= 0.0) return event.setToCoalescence(active_node(i), i);
     }
 
-    // Migration and Recombination only on time line 0    
+    // Migration and Recombination only on time line 0
     if (time_line != 0) continue;
 
     // Recombination
@@ -705,7 +710,7 @@ void Forest::sampleEventType(const double time, const size_t time_line,
       for ( size_t j = 0; j < model().population_number(); ++j) {
         sample -= model().migration_rate(active_node(i)->population(), j);
         if ( sample <= 0.0 ) return event.setToMigration(active_node(i), i, j);
-      } 
+      }
       throw std::logic_error("Error Sampling Type of Event");
     }
     sample -= model().total_migration_rate(active_node(i)->population());
@@ -714,7 +719,7 @@ void Forest::sampleEventType(const double time, const size_t time_line,
   // If we are here, than we should have sampled a pw coalescence...
   assert( states_[0] == 1 && states_[1] == 1 );
   assert( active_nodes_[0]->population() == active_nodes_[1]->population() );
-  assert( sample <= calcPwCoalescenceRate(active_nodes_[0]->population(), ti) );  
+  assert( sample <= calcPwCoalescenceRate(active_nodes_[0]->population(), ti) );
   return event.setToPwCoalescence();
 }
 
@@ -725,12 +730,12 @@ void Forest::sampleEventType(const double time, const size_t time_line,
  * the time_line counter.
  *
  * \param new_time An ExpoLimit Sample
- * \param time_line The timeline that the sample was from 
- * \param current_time The variable that save the time of the nearest event 
- * \param time_line The variable that saves the timeline of the nearest event 
- * \return Nothing, but updates current_time and current_time_line   
+ * \param time_line The timeline that the sample was from
+ * \param current_time The variable that save the time of the nearest event
+ * \param time_line The variable that saves the timeline of the nearest event
+ * \return Nothing, but updates current_time and current_time_line
  */
-void Forest::selectFirstTime(const double new_time, const size_t time_line, 
+void Forest::selectFirstTime(const double new_time, const size_t time_line,
                              double &current_time, size_t &current_time_line) const {
   if (new_time == -1) return;
   if (current_time == -1 || new_time < current_time) {
@@ -755,8 +760,8 @@ size_t Forest::getNodeState(Node const *node, const double current_time) const {
   if (node->is_root()) return(1);
   if (!node->local()) return(2);
   dout << "Error getting node state." << std::endl;
-  dout << "Height: " << node->height() 
-      << " current time: " << current_time 
+  dout << "Height: " << node->height()
+      << " current time: " << current_time
       << " diff: " << node->height() - current_time << std::endl;
   dout << "Node local: " << node->local() << std::endl;
   dout << "Node root: " << node->is_root() << std::endl;
@@ -788,8 +793,8 @@ double Forest::calcRecombinationRate(Node const* node) const {
   } else {
     // Rec rate may change. Accumulate the total rate.
 
-    double rate = model().recombination_rate() * 
-        (this->current_base() - model().getCurrentSequencePosition()); 
+    double rate = model().recombination_rate() *
+        (this->current_base() - model().getCurrentSequencePosition());
     size_t idx = model().get_position_index() - 1;
 
     while (model().change_position(idx) > last_update_pos) {
@@ -804,7 +809,7 @@ double Forest::calcRecombinationRate(Node const* node) const {
 }
 
 
-/** 
+/**
  * Even if no event occurred in a time interval, there is some stuff that we
  * might have to do. Mainly moving the "active" flag upwards if a active node
  * was looking for recombinations, but none occurred. In that case, we also
@@ -812,11 +817,27 @@ double Forest::calcRecombinationRate(Node const* node) const {
  *
  * \param ti The current time interval
  * \param coalescences_finished temp variable to pass the information that the
- *    coalescence has finished. 
+ *    coalescence has finished.
  */
 void Forest::implementNoEvent(const TimeInterval &ti, bool &coalescence_finished) {
-  if (ti.end_height() == DBL_MAX) 
-    throw std::logic_error("Lines did not coalescence. If you use an negative growth parameter (population rapidly declining forward in time), you need to set it to a non-negative value at some time.");
+  if (ti.end_height() == DBL_MAX) {
+    std::stringstream message;
+    message << "Lines did not coalescence." << std::endl;
+    if (active_node(0)->population() != active_node(1)->population()) {
+      message << "The lines were in populations " << active_node(0)->population() + 1
+          << " and " << active_node(1)->population() + 1 << "." << std::endl
+          << "You should add on opportunity for migration between these populations."
+          << std::endl;
+    }
+
+    else if (model().growth_rate(active_node(0)->population())) {
+        message << "Population " << active_node(0)->population() + 1
+                << " has a negative growth factor for infinite time." << std::endl
+                << "This can prevent coalescence. " << std::endl;
+    }
+
+    throw std::logic_error(message.str());
+  }
   if (states_[0] == 2) {
     set_active_node(0, possiblyMoveUpwards(active_node(0), ti));
     if (active_node(0)->local()) {
@@ -825,8 +846,8 @@ void Forest::implementNoEvent(const TimeInterval &ti, bool &coalescence_finished
       updateAbove(active_node(0));
       coalescence_finished = true;
       tmp_event_time_ = active_node(0)->height();
-      contemporaries_.replace(active_node(0), 
-                              active_node(0)->first_child(), 
+      contemporaries_.replace(active_node(0),
+                              active_node(0)->first_child(),
                               active_node(0)->second_child());
       return;
     }
@@ -837,7 +858,7 @@ void Forest::implementNoEvent(const TimeInterval &ti, bool &coalescence_finished
   if (states_[1] == 2) set_active_node(1, possiblyMoveUpwards(active_node(1), ti));
 
   if (active_node(0) == active_node(1)) {
-    dout << "* * * Active Nodes hit each other in " << active_node(0) 
+    dout << "* * * Active Nodes hit each other in " << active_node(0)
         << ". Done." << std::endl;
     updateAbove(active_node(0));
     coalescence_finished = true;
@@ -865,7 +886,7 @@ void Forest::implementCoalescence(const Event &event, TimeIntervalIterator &tii)
   Node* target = contemporaries_.sample(coal_node->population());
 
   dout << "* * * Above node " << target << std::endl;
-  assert( target->height() < event.time() ); 
+  assert( target->height() < event.time() );
   assert( coal_node->population() == target->population() );
   assert( getEventNode() != NULL );
   assert( getOtherNode() != NULL );
@@ -876,7 +897,7 @@ void Forest::implementCoalescence(const Event &event, TimeIntervalIterator &tii)
   // Actually implement the coalescence
   // ---------------------------------------------
 
-  // Look if we can reuse the root that coalesced for marking the point of 
+  // Look if we can reuse the root that coalesced for marking the point of
   // coalescence
   if ( coal_node->countChildren() == 1 && !coal_node->is_migrating() ){
     assert( coal_node->local() );
@@ -894,7 +915,7 @@ void Forest::implementCoalescence(const Event &event, TimeIntervalIterator &tii)
 
   // Now we have:
   // target:    Node in the target tree under the coalescence
-  // coal_node: Root of the coalescing tree 
+  // coal_node: Root of the coalescing tree
   // new_node:  New parent of 'target' and 'coal_node'
 
   // Update new_node
@@ -914,7 +935,7 @@ void Forest::implementCoalescence(const Event &event, TimeIntervalIterator &tii)
 
   // And update
   coal_node->make_local();
-  updateAbove(coal_node, false, false); 
+  updateAbove(coal_node, false, false);
 
   set_active_node(event.active_node_nr(), new_node);
 
@@ -924,7 +945,7 @@ void Forest::implementCoalescence(const Event &event, TimeIntervalIterator &tii)
   // ---------------------------------------------
 
   if ( getOtherNodesState() == 2 ) {
-    // If the coalescing node coalesced into the branch directly above 
+    // If the coalescing node coalesced into the branch directly above
     // a recombining node, we are done.
     if ( getOtherNode()->parent() == getEventNode() ) {
       dout << "* * * Recombining Node moved into coalesced node. Done." << std::endl;
@@ -940,15 +961,15 @@ void Forest::implementCoalescence(const Event &event, TimeIntervalIterator &tii)
   }
 
   if ( target->local() ) {
-    // Only active_node(0) can coalescence into local nodes. active_node(1) is 
-    // at least the local root and hence above all local nodes. 
+    // Only active_node(0) can coalescence into local nodes. active_node(1) is
+    // at least the local root and hence above all local nodes.
     // If active_node(0) coalescences into the local tree, there are no more
-    // active nodes and we are done. 
+    // active nodes and we are done.
     assert( event.active_node_nr() == 0 );
     assert( states_[1] == 0 );
 
     dout << "* * * We hit the local tree. Done." << std::endl;
-    updateAbove(getEventNode()); 
+    updateAbove(getEventNode());
     coalescence_finished_ = true;
     contemporaries_.replace(new_node, coal_node, target);
     return;
@@ -962,9 +983,9 @@ void Forest::implementCoalescence(const Event &event, TimeIntervalIterator &tii)
 
 
 
-/** 
+/**
  * @brief Modifies the forest to reflect that two coalescing nodes coalesced together.
- * 
+ *
  * @param root_1 The first coalescing node
  * @param root_2 The second coalescing node
  * @param time   The time at which the coalescence happens
@@ -998,7 +1019,7 @@ void Forest::implementPwCoalescence(Node* root_1, Node* root_2, const double tim
     new_root = root_1;
     root_1 = root_1->first_child();
     assert( root_1 != NULL );
-  } 
+  }
   else if (root_2->countChildren() == 1 && !root_2->is_migrating()) {
     // only root_2 has a single branch => use as new root
     this->nodes()->move(root_2, time);
@@ -1044,8 +1065,8 @@ void Forest::implementMigration(const Event &event, const bool &recalculate, Tim
   dout << "* * Implementing migration event... " << std::flush;
   assert( event.node()->is_root() );
 
-  // There is only little to do if we can reuse the event.node() 
-  if ( event.node()->is_unimportant() || 
+  // There is only little to do if we can reuse the event.node()
+  if ( event.node()->is_unimportant() ||
        ( event.node()->height() == event.time() && event.node()->is_migrating() ) ) {
     dout << "Reusing: " << event.node() << "... " << std::flush;
     nodes()->move(event.node(), event.time());
@@ -1054,7 +1075,7 @@ void Forest::implementMigration(const Event &event, const bool &recalculate, Tim
   } else {
     // Otherwise create a new node that marks the migration event,
     Node* mig_node = nodes()->createNode(event.time());
-    dout << "Marker: " << mig_node << "... " << std::flush; 
+    dout << "Marker: " << mig_node << "... " << std::flush;
     nodes()->add(mig_node, event.node());
     mig_node->set_population(event.mig_pop());
 
@@ -1082,40 +1103,33 @@ void Forest::implementMigration(const Event &event, const bool &recalculate, Tim
 void Forest::implementFixedTimeEvent(TimeIntervalIterator &ti) {
   dout << "* * Fixed time event" << std::endl;
   double sample;
-  bool migrated;
-  size_t chain_cnt, pop_number = model().population_number();
+  auto mig_events = model().single_mig_events();
 
   for (size_t i = 0; i < 2; ++i) {
     if (states_[i] != 1) continue;
-    chain_cnt = 0;
-    while (true) {
-      migrated = false;
-      sample = random_generator()->sample();
-
-      for (size_t j = 0; j < pop_number; ++j) {
-        sample -= model().single_mig_pop(active_node(i)->population(), j);
-        if (sample < 0) {
-          tmp_event_ = Event((*ti).start_height());
-          tmp_event_.setToMigration(active_node(i), i, j);
-          implementMigration(tmp_event_, false, ti);
-          migrated = true;
-          break;
-        }
+    
+    sample = random_generator()->sample();
+    for (auto me : mig_events) {
+      if (active_node(i)->population() == me.source_pop) {
+        sample -= me.prob;
       }
 
-      // Stop if no migration occurred
-      if (!migrated) break;
-
-      // Resolve a maximum of 10k chained events for each node
-      if (chain_cnt == 10000) 
-        throw std::logic_error("Circle detected when moving individuals between populations");
-      ++chain_cnt;
+      if (sample < 0) {
+        dout << "* * * a" << i << ": Migration from " 
+             << me.source_pop << " to " 
+             << me.sink_pop << std::endl;
+        tmp_event_ = Event((*ti).start_height());
+        tmp_event_.setToMigration(active_node(i), i, me.sink_pop);
+        implementMigration(tmp_event_, false, ti);
+        sample = random_generator()->sample();
+      }
     }
   }
+
   assert( printTree() );
 }
 
-/** 
+/**
  * Helper function for doing a coalescence.
  * Moves the 'active' flag (i.e. the node stored in root_1 or root_2 in sampleCoalescence)
  * from a node to it's parent if the branch above the node
@@ -1130,7 +1144,7 @@ void Forest::implementFixedTimeEvent(TimeIntervalIterator &ti) {
  *
  * \param node An active node
  * \param time_interval The time interval the coalescence is currently in.
- * 
+ *
  * \return  Either the parent of 'node' if we need to move upwards or 'node'
  *          itself
  */
@@ -1159,7 +1173,7 @@ bool Forest::pruneNodeIfNeeded(Node* node, const bool prune_orphans) {
       // of the current coalescence.
       if (node == primary_root()) set_primary_root(NULL);
       nodes()->remove(node);
-      return true; 
+      return true;
     }
     // Other roots stay
     return false;
@@ -1171,20 +1185,20 @@ bool Forest::pruneNodeIfNeeded(Node* node, const bool prune_orphans) {
       assert(!node->is_root());
 
       node->parent()->change_child(node, NULL);
-      if (node->countChildren() == 0) nodes()->remove(node); 
-      else { 
+      if (node->countChildren() == 0) nodes()->remove(node);
+      else {
         // Remove link between `node` and its parent,
-        // which effectively removes the branch, 
+        // which effectively removes the branch,
         // and separates the subtree below from the main tree
         Node* parent = node->parent();
         node->set_parent(NULL);
         updateAbove(parent, false, true, true);
       }
       return true;
-    } 
+    }
 
     else if (node->countChildren() == 1 && !node->is_migrating()) {
-      // Unneeded nodes 
+      // Unneeded nodes
       dout << "* * * PRUNING: Removing node " << node << " from tree (unneeded)" << std::endl;
       assert(!node->is_migrating());
       assert(node->first_child()->last_update() == node->last_update());
@@ -1192,7 +1206,7 @@ bool Forest::pruneNodeIfNeeded(Node* node, const bool prune_orphans) {
 
       Node* child = node->first_child();
       child->set_parent(node->parent());
-      node->parent()->change_child(node, child); 
+      node->parent()->change_child(node, child);
       nodes()->remove(node);
       return true;
     }
@@ -1208,18 +1222,28 @@ void Forest::calcSegmentSumStats() {
   }
 }
 
+
 void Forest::clearSumStats() {
   for (size_t i = 0; i < model().countSummaryStatistics(); ++i) {
     model().getSummaryStatistic(i)->clear();
   }
 }
 
+
 void Forest::printLocusSumStats(std::ostream &output) const {
   for (size_t i = 0; i < model().countSummaryStatistics(); ++i) {
     model().getSummaryStatistic(i)->printLocusOutput(output);
   }
 }
 
+
+void Forest::printSegmentSumStats(std::ostream &output) const {
+  for (size_t i = 0; i < model().countSummaryStatistics(); ++i) {
+    model().getSummaryStatistic(i)->printSegmentOutput(output);
+  }
+}
+
+
 void Forest::clear() {
   // Clear roots tracking
   set_local_root(NULL);
@@ -1227,6 +1251,7 @@ void Forest::clear() {
 
   // Clear nodes
   nodes()->clear();
+  contemporaries()->clear(true);
 
   // Reset Position & Segment Counts
   this->rec_bases_.clear();
@@ -1235,7 +1260,7 @@ void Forest::clear() {
 
   // Clear Summary Statistics
   this->clearSumStats();
-  
+
   // Reset Model
   writable_model()->resetTime();
   writable_model()->resetSequencePosition();
@@ -1330,5 +1355,8 @@ void Forest::sampleNextBase() {
     // Recombination in the sequence segment
     set_next_base(current_base() + length);
   }
-} 
+
+  assert(next_base() > current_base());
+  assert(next_base() <= model().loci_length());
+}
 
