@@ -50,8 +50,12 @@ Model Param::parse() {
   bool tmrca = false,
        newick_trees = false,
        orientedForest = false,
-       sfs = false; 
+       sfs = false,
+       transpose = false;
 
+  // Tracks if demographic where added to the model.
+  // After the first demographic feature, defining substructure is no longer allowed.
+  bool has_demographic_feature = false;
 
   // The minimal time at which -eM, -eN, -eG, -eI, -ema and -es are allowed to happen. Is
   // increased by using -es.
@@ -114,6 +118,11 @@ Model Param::parse() {
     // ------------------------------------------------------------------
     // Set number of subpopulations and samples at time 0
     else if (*argv_i == "-I") {
+      // Check that -I is used immediately of the first two mandatory arguments
+      if (has_demographic_feature) {
+        throw std::invalid_argument("Option '-I' must be used before demographic '-M', '-N' or '-G' is used.");
+      }
+
       model.set_population_number(readNextInt());
       std::vector<size_t> sample_size;
       for (size_t i = 0; i < model.population_number(); ++i) {
@@ -123,7 +132,7 @@ Model Param::parse() {
       // there might or might not follow a symmetric migration rate
       try {
         model.addSymmetricMigration(0.0, readNextInput<double>()/(model.population_number()-1), true, true);
-      } catch (std::invalid_argument e) {
+      } catch (std::invalid_argument& e) {
         --argv_i;
       }
     }
@@ -146,6 +155,7 @@ Model Param::parse() {
     // Populations sizes 
     // ------------------------------------------------------------------
     else if (*argv_i == "-eN" || *argv_i == "-N") {
+      has_demographic_feature = true;
       if (*argv_i == "-eN") time = readNextInput<double>();
       else time = 0.0;
       if (time < min_time) {
@@ -168,6 +178,7 @@ Model Param::parse() {
     // Exponential Growth 
     // ------------------------------------------------------------------
     else if (*argv_i == "-G" || *argv_i == "-eG") {
+      has_demographic_feature = true;
       if (*argv_i == "-eG") time = readNextInput<double>();
       else time = 0.0;
       if (time < min_time) {
@@ -225,6 +236,7 @@ Model Param::parse() {
     }
 
     else if (*argv_i == "-M" || *argv_i == "-eM") {
+      has_demographic_feature = true;
       if (*argv_i == "-eM") {
         time = readNextInput<double>();
       }
@@ -233,6 +245,9 @@ Model Param::parse() {
         throw std::invalid_argument(std::string("If you use '-M' or '-eM' in a model with population merges ('-es'),") +
                                     std::string("then you need to sort both arguments by time."));
       }
+      if (model.population_number() == 1) {
+        throw std::invalid_argument("-M and -eM options can not be used if there is just one population");
+      }
       model.addSymmetricMigration(time, readNextInput<double>()/(model.population_number()-1), true, true);
     }
 
@@ -367,7 +382,7 @@ Model Param::parse() {
       try {
         // Maybe read in up to 3 seeds (ms compatibility)
         for (size_t i = 1; i < 3; ++i) seeds.at(i) = readNextInt();
-      } catch (std::invalid_argument e) {
+      } catch (std::invalid_argument& e) {
         --argv_i;
       }
 
@@ -394,7 +409,11 @@ Model Param::parse() {
 
     else if (*argv_i == "-print-model" || *argv_i == "--print-model") {
       this->set_print_model(true);
-    } 
+    }
+
+    else if (*argv_i == "-transpose-segsites" || *argv_i == "--transpose-segsites") {
+      transpose = true;
+    }
 
 
     // ------------------------------------------------------------------
@@ -427,10 +446,12 @@ Model Param::parse() {
   if (newick_trees) model.addSummaryStatistic(std::make_shared<NewickTree>(this->precision(), 
                                                                            model.has_recombination()));
   if (orientedForest) {
-    model.addSummaryStatistic(std::make_shared<OrientedForest>(model.sample_size(), this->precision()));
+    if (newick_trees) throw std::invalid_argument("scrm does not support '-T' and '-O' at the same time"); 
+    model.addSummaryStatistic(std::make_shared<OrientedForest>(model.sample_size()));
   }
   if (tmrca) model.addSummaryStatistic(std::make_shared<TMRCA>());
   if (seg_sites.get() != NULL) model.addSummaryStatistic(seg_sites);
+  if (seg_sites.get() != NULL && transpose ) seg_sites->set_transpose(true);
   if (sfs) {
     if (seg_sites == NULL) 
       throw std::invalid_argument("You need to give a mutation rate ('-t') to simulate a SFS"); 
